#    Copyright 2016 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#


def generate_timeout_series(timeout):
    """Generate a series of times that exceeds the given timeout.

    Yields a series of fake time.time() floating point numbers
    such that the difference between each pair in the series just
    exceeds the timeout value that is passed in.  Useful for
    mocking time.time() in methods that otherwise wait for timeout
    seconds.
    """
    iteration = 0
    while True:
        iteration += 1
        yield (iteration * timeout) + iteration



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

show_qos = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'qos_specs': {
                'type': 'object',
                'properties': {
                    'name': {'type': 'string'},
                    'id': {'type': 'string', 'format': 'uuid'},
                    'consumer': {'type': 'string'},
                    'specs': {'type': ['object', 'null']},
                },
                'additionalProperties': False,
                'required': ['name', 'id', 'specs']
            },
            'links': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'href': {'type': 'string',
                                 'format': 'uri'},
                        'rel': {'type': 'string'},
                    },
                    'additionalProperties': False,
                    'required': ['href', 'rel']
                }
            }
        },
        'additionalProperties': False,
        'required': ['qos_specs', 'links']
    }
}

delete_qos = {'status_code': [202]}

list_qos = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'qos_specs': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'specs': {
                            'type': 'object',
                            'patternProperties': {'^.+$': {'type': 'string'}}
                        },
                        'consumer': {'type': 'string'},
                        'id': {'type': 'string', 'format': 'uuid'},
                        'name': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['specs', 'id', 'name']
                }
            }
        },
        'additionalProperties': False,
        'required': ['qos_specs']
    }
}

set_qos_key = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'qos_specs': {
                'type': 'object',
                'patternProperties': {'^.+$': {'type': 'string'}}
            },
        },
        'additionalProperties': False,
        'required': ['qos_specs']
    }
}

unset_qos_key = {'status_code': [202]}
associate_qos = {'status_code': [202]}

show_association_qos = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'qos_associations': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'association_type': {'type': 'string'},
                        'id': {'type': 'string', 'format': 'uuid'},
                        'name': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['association_type', 'id', 'name']
                }
            },
        },
        'additionalProperties': False,
        'required': ['qos_associations']
    }
}

disassociate_qos = {'status_code': [202]}
disassociate_all_qos = {'status_code': [202]}



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class ProjectTagsClient(rest_client.RestClient):
    api_version = "v3"

    def update_project_tag(self, project_id, tag):
        """Updates the specified tag and adds it to the project's list of tags.

        """
        url = 'projects/%s/tags/%s' % (project_id, tag)
        resp, body = self.put(url, '{}')
        # NOTE(felipemonteiro): This API endpoint returns 201 AND an empty
        # response body, which is consistent with the spec:
        # https://specs.openstack.org/openstack/api-wg/guidelines/tags.html#addressing-individual-tags
        self.expected_success(201, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_project_tags(self, project_id):
        """List tags for a project."""
        url = "projects/%s/tags" % project_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_all_project_tags(self, project_id, tags, **kwargs):
        """Updates all the tags for a project.

        Any existing tags not specified will be deleted.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#modify-tag-list-for-a-project
        """
        body = {'tags': tags}
        if kwargs:
            body.update(kwargs)
        put_body = json.dumps(body)
        resp, body = self.put('projects/%s/tags' % project_id, put_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def check_project_tag_existence(self, project_id, tag):
        """Check if a project contains a tag."""
        url = 'projects/%s/tags/%s' % (project_id, tag)
        resp, body = self.get(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_project_tag(self, project_id, tag):
        """Delete a project tag."""
        url = 'projects/%s/tags/%s' % (project_id, tag)
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_all_project_tags(self, project_id):
        """Delete all tags from a project."""
        resp, body = self.delete('projects/%s/tags' % project_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

get_pools_no_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'pools': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'name': {'type': 'string'},
                    },
                    'additionalProperties': False,
                    'required': ['name']
                }
            }
        },
        'additionalProperties': False,
        'required': ['pools'],
    }
}

get_pools_with_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'pools': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'name': {'type': 'string'},
                        'capabilities': {
                            'type': ['object', 'null'],
                            'properties': {
                                'updated': parameter_types.date_time_or_null,
                                'QoS_support': {'type': 'boolean'},
                                'total_capacity_gb': {
                                    'type': ['number', 'string']
                                },
                                'volume_backend_name': {'type': 'string'},
                                'free_capacity_gb': {
                                    'type': ['number', 'string']
                                },
                                'driver_version': {'type': 'string'},
                                'reserved_percentage': {'type': 'integer'},
                                'storage_protocol': {'type': 'string'},
                                'vendor_name': {'type': 'string'},
                                'timestamp': parameter_types.date_time_or_null
                            },
                            # Because some legacy volumes or backends may not
                            # support pools, so no required fields here.
                        },
                    },
                    'additionalProperties': False,
                    'required': ['name', 'capabilities']
                }
            }
        },
        'additionalProperties': False,
        'required': ['pools'],
    }
}



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_8 import servers

get_server = copy.deepcopy(servers.get_server)
get_server['response_body']['properties']['server'][
    'properties'].update({'locked': {'type': 'boolean'}})
get_server['response_body']['properties']['server'][
    'required'].append('locked')

list_servers_detail = copy.deepcopy(servers.list_servers_detail)
list_servers_detail['response_body']['properties']['servers']['items'][
    'properties'].update({'locked': {'type': 'boolean'}})
list_servers_detail['response_body']['properties']['servers']['items'][
    'required'].append('locked')

update_server = copy.deepcopy(servers.update_server)
update_server['response_body']['properties']['server'][
    'properties'].update({'locked': {'type': 'boolean'}})
update_server['response_body']['properties']['server'][
    'required'].append('locked')

rebuild_server = copy.deepcopy(servers.rebuild_server)
rebuild_server['response_body']['properties']['server'][
    'properties'].update({'locked': {'type': 'boolean'}})
rebuild_server['response_body']['properties']['server'][
    'required'].append('locked')

rebuild_server_with_admin_pass = copy.deepcopy(
    servers.rebuild_server_with_admin_pass)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'locked': {'type': 'boolean'}})
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'required'].append('locked')

# NOTE: Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.8 ******
list_servers = copy.deepcopy(servers.list_servers)
show_server_diagnostics = copy.deepcopy(servers.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers.get_remote_consoles)
attach_volume = copy.deepcopy(servers.attach_volume)
show_volume_attachment = copy.deepcopy(servers.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers.list_volume_attachments)



# Copyright 2013 Huawei Technologies Co.,LTD.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class HostsAdminNegativeTestJSON(base.BaseV2ComputeAdminTest):
    """Tests hosts API using admin privileges."""

    max_microversion = '2.42'

    @classmethod
    def setup_clients(cls):
        super(HostsAdminNegativeTestJSON, cls).setup_clients()
        cls.client = cls.os_admin.hosts_client
        cls.non_admin_client = cls.os_primary.hosts_client

    @classmethod
    def resource_setup(cls):
        super(HostsAdminNegativeTestJSON, cls).resource_setup()
        hosts = cls.client.list_hosts()['hosts']
        if not hosts:
            raise lib_exc.NotFound("no host found")
        cls.hostname = hosts[0]['host_name']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('dd032027-0210-4d9c-860e-69b1b8deed5f')
    def test_list_hosts_with_non_admin_user(self):
        """Non admin user is not allowed to list hosts"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.list_hosts)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e75b0a1a-041f-47a1-8b4a-b72a6ff36d3f')
    def test_show_host_detail_with_nonexistent_hostname(self):
        """Showing host detail with not existing hostname should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_host, 'nonexistent_hostname')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('19ebe09c-bfd4-4b7c-81a2-e2e0710f59cc')
    def test_show_host_detail_with_non_admin_user(self):
        """Non admin user is not allowed to show host details"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.show_host,
                          self.hostname)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e40c72b1-0239-4ed6-ba21-81a184df1f7c')
    def test_update_host_with_non_admin_user(self):
        """Non admin user is not allowed to update host"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.update_host,
                          self.hostname,
                          status='enable',
                          maintenance_mode='enable')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('fbe2bf3e-3246-4a95-a59f-94e4e298ec77')
    def test_update_host_with_invalid_status(self):
        """Updating host to invalid status should fail

        'status' can only be 'enable' or 'disable'.
        """
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_host,
                          self.hostname,
                          status='invalid',
                          maintenance_mode='enable')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ab1e230e-5e22-41a9-8699-82b9947915d4')
    def test_update_host_with_invalid_maintenance_mode(self):
        """Updating host to invalid maintenance mode should fail

        'maintenance_mode' can only be 'enable' or 'disable'.
        """
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_host,
                          self.hostname,
                          status='enable',
                          maintenance_mode='invalid')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0cd85f75-6992-4a4a-b1bd-d11e37fd0eee')
    def test_update_host_without_param(self):
        """Updating host without param should fail

        'status' or 'maintenance_mode' is needed for host update
        """
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_host,
                          self.hostname)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('23c92146-2100-4d68-b2d6-c7ade970c9c1')
    def test_update_nonexistent_host(self):
        """Updating not existing host should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.update_host,
                          'nonexistent_hostname',
                          status='enable',
                          maintenance_mode='enable')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0d981ac3-4320-4898-b674-82b61fbb60e4')
    def test_startup_nonexistent_host(self):
        """Starting up not existing host should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.startup_host,
                          'nonexistent_hostname')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9f4ebb7e-b2ae-4e5b-a38f-0fd1bb0ddfca')
    def test_startup_host_with_non_admin_user(self):
        """Non admin user is not allowed to startup host"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.startup_host,
                          self.hostname)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9e637444-29cf-4244-88c8-831ae82c31b6')
    def test_shutdown_nonexistent_host(self):
        """Shutting down not existing host should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.shutdown_host,
                          'nonexistent_hostname')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a803529c-7e3f-4d3c-a7d6-8e1c203d27f6')
    def test_shutdown_host_with_non_admin_user(self):
        """Non admin user is not allowed to shutdown host"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.shutdown_host,
                          self.hostname)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f86bfd7b-0b13-4849-ae29-0322e83ee58b')
    def test_reboot_nonexistent_host(self):
        """Rebooting not existing host should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.reboot_host,
                          'nonexistent_hostname')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('02d79bb9-eb57-4612-abf6-2cb38897d2f8')
    def test_reboot_host_with_non_admin_user(self):
        """Non admin user is not allowed to reboot host"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.reboot_host,
                          self.hostname)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class RoutersClient(base.BaseNetworkClient):

    def create_router(self, **kwargs):
        """Create a router.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-router
        """
        post_body = {'router': kwargs}
        uri = '/routers'
        return self.create_resource(uri, post_body)

    def update_router(self, router_id, **kwargs):
        """Updates a logical router.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-router
        """
        uri = '/routers/%s' % router_id
        update_body = {'router': kwargs}
        return self.update_resource(uri, update_body)

    def show_router(self, router_id, **fields):
        """Shows details for a router.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-router-details
        """
        uri = '/routers/%s' % router_id
        return self.show_resource(uri, **fields)

    def delete_router(self, router_id):
        uri = '/routers/%s' % router_id
        return self.delete_resource(uri)

    def list_routers(self, **filters):
        """Lists logical routers.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-routers
        """
        uri = '/routers'
        return self.list_resources(uri, **filters)

    def add_router_interface(self, router_id, **kwargs):
        """Add router interface.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#add-interface-to-router
        """
        uri = '/routers/%s/add_router_interface' % router_id
        return self.update_resource(uri, kwargs)

    def remove_router_interface(self, router_id, **kwargs):
        """Remove router interface.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#remove-interface-from-router
        """
        uri = '/routers/%s/remove_router_interface' % router_id
        return self.update_resource(uri, kwargs)

    def list_l3_agents_hosting_router(self, router_id):
        uri = '/routers/%s/l3-agents' % router_id
        return self.list_resources(uri)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import custom_matchers
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as lib_exc
import tempest.test

CONF = config.CONF


def delete_containers(containers, container_client, object_client):
    """Remove containers and all objects in them.

    The containers should be visible from the container_client given.
    Will not throw any error if the containers don't exist.
    Will not check that object and container deletions succeed.
    After delete all the objects from a container, it will wait 2
    seconds before delete the container itself, in order to deployments
    using HA proxy sync the deletion properly, otherwise, the container
    might fail to be deleted because it's not empty.

    :param containers: List of containers(or string of a container)
                       to be deleted
    :param container_client: Client to be used to delete containers
    :param object_client: Client to be used to delete objects
    """
    if isinstance(containers, str):
        containers = [containers]

    for cont in containers:
        try:
            params = {'limit': 9999, 'format': 'json'}
            _, objlist = container_client.list_container_objects(cont, params)
            # delete every object in the container
            for obj in objlist:
                object_client.delete_object(cont, obj['name'])
                object_client.wait_for_resource_deletion(obj['name'], cont)
            # Verify resource deletion
            container_client.delete_container(cont)
            container_client.wait_for_resource_deletion(cont)
        except lib_exc.NotFound:
            pass


class BaseObjectTest(tempest.test.BaseTestCase):

    credentials = [['operator', CONF.object_storage.operator_role]]

    @classmethod
    def skip_checks(cls):
        super(BaseObjectTest, cls).skip_checks()
        if not CONF.service_available.swift:
            skip_msg = ("%s skipped as swift is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources()
        super(BaseObjectTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BaseObjectTest, cls).setup_clients()
        cls.object_client = cls.os_roles_operator.object_client
        cls.bulk_client = cls.os_roles_operator.bulk_client
        cls.container_client = cls.os_roles_operator.container_client
        cls.account_client = cls.os_roles_operator.account_client
        cls.capabilities_client = cls.os_roles_operator.capabilities_client

    @classmethod
    def resource_setup(cls):
        super(BaseObjectTest, cls).resource_setup()

        # Make sure we get fresh auth data after assigning swift role
        cls.object_client.auth_provider.clear_auth()
        cls.container_client.auth_provider.clear_auth()
        cls.account_client.auth_provider.clear_auth()

        # make sure that discoverability is enabled and that the sections
        # have not been disallowed by Swift
        cls.policies = None

        if CONF.object_storage_feature_enabled.discoverability:
            body = cls.capabilities_client.list_capabilities()

            if 'swift' in body and 'policies' in body['swift']:
                cls.policies = body['swift']['policies']

        cls.containers = []

    @classmethod
    def create_container(cls):
        # wrapper that returns a test container
        container_name = data_utils.rand_name(name='TestContainer')
        cls.container_client.update_container(container_name)
        cls.containers.append(container_name)

        return container_name

    @classmethod
    def create_object(cls, container_name, object_name=None,
                      data=None, metadata=None):
        # wrapper that returns a test object
        if object_name is None:
            object_name = data_utils.rand_name(name='TestObject')
        if data is None:
            data = data_utils.random_bytes()
        cls.object_client.create_object(container_name,
                                        object_name,
                                        data,
                                        metadata=metadata)

        return object_name, data

    @classmethod
    def delete_containers(cls, container_client=None, object_client=None):
        if container_client is None:
            container_client = cls.container_client
        if object_client is None:
            object_client = cls.object_client
        delete_containers(cls.containers, container_client, object_client)

    def assertHeaders(self, resp, target, method):
        """Check the existence and the format of response headers"""

        self.assertThat(resp, custom_matchers.ExistsAllResponseHeaders(
                        target, method, self.policies))
        self.assertThat(resp, custom_matchers.AreAllWellFormatted())



# Copyright 2016 VMware, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.network import base
from tempest.lib import decorators


class NetworksApiDiscovery(base.BaseNetworkTest):
    @decorators.attr(type='smoke')
    @decorators.idempotent_id('cac8a836-c2e0-4304-b556-cd299c7281d1')
    def test_api_version_resources(self):
        """Test that GET / returns expected resources.

        The versions document returned by Neutron returns a few other
        resources other than just available API versions: it also
        states the status of each API version and provides links to
        schema.
        """

        result = self.network_versions_client.list_versions()
        expected_versions = ('v2.0',)
        expected_resources = ('id', 'links', 'status')
        received_list = result.values()

        for item in received_list:
            for version in item:
                for resource in expected_resources:
                    self.assertIn(resource, version)
                self.assertIn(version['id'], expected_versions)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('e64b7216-3178-4263-967c-d389290988bf')
    def test_show_api_v2_details(self):
        """Test that GET /v2.0/ returns expected resources."""
        current_version = 'v2.0'
        expected_resources = ('subnet', 'network', 'port')
        result = self.network_versions_client.show_version(current_version)
        actual_resources = [r['name'] for r in result['resources']]
        for resource in expected_resources:
            self.assertIn(resource, actual_resources)



# Copyright 2016 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from unittest import mock

from tempest.lib import exceptions
from tempest.lib.services.object_storage import object_client
from tempest.tests import base
from tempest.tests.lib import fake_auth_provider


class TestObjectClient(base.TestCase):

    def setUp(self):
        super(TestObjectClient, self).setUp()
        self.fake_auth = fake_auth_provider.FakeAuthProvider()
        self.url = self.fake_auth.base_url(None)
        self.object_client = object_client.ObjectClient(self.fake_auth,
                                                        'swift', 'region1')

    @mock.patch('tempest.lib.services.object_storage.object_client.'
                'ObjectClient._create_connection')
    def test_create_object_continue_no_data(self, mock_poc):
        self._validate_create_object_continue(None, mock_poc)

    @mock.patch('tempest.lib.services.object_storage.object_client.'
                'ObjectClient._create_connection')
    def test_create_object_continue_with_data(self, mock_poc):
        self._validate_create_object_continue('hello', mock_poc)

    @mock.patch('tempest.lib.services.object_storage.object_client.'
                'ObjectClient._create_connection')
    def test_create_continue_with_no_continue_received(self, mock_poc):
        self._validate_create_object_continue('hello', mock_poc,
                                              initial_status=201)

    def _validate_create_object_continue(self, req_data,
                                         mock_poc, initial_status=100):

        expected_hdrs = {
            'X-Auth-Token': self.fake_auth.get_token(),
            'content-length': 0 if req_data is None else len(req_data),
            'Expect': '100-continue'}

        # Setup the Mocks prior to invoking the object creation
        mock_resp_cls = mock.Mock()
        mock_resp_cls._read_status.return_value = ("1", initial_status, "OK")

        mock_poc.return_value.response_class.return_value = mock_resp_cls

        # This is the final expected return value
        mock_poc.return_value.getresponse.return_value.status = 201
        mock_poc.return_value.getresponse.return_value.reason = 'OK'

        # Call method to PUT object using expect:100-continue
        cnt = "container1"
        obj = "object1"
        path = "/%s/%s" % (cnt, obj)

        # If the expected initial status is not 100, then an exception
        # should be thrown and the connection closed
        if initial_status == 100:
            status, reason = \
                self.object_client.create_object_continue(cnt, obj, req_data)
        else:
            self.assertRaises(exceptions.UnexpectedResponseCode,
                              self.object_client.create_object_continue, cnt,
                              obj, req_data)
            mock_poc.return_value.close.assert_called_once_with()

        # Verify that putrequest is called 1 time with the appropriate values
        mock_poc.return_value.putrequest.assert_called_once_with('PUT', path)

        # Verify that headers were written, including "Expect:100-continue"
        calls = []

        for header, value in expected_hdrs.items():
            calls.append(mock.call(header, value))

        mock_poc.return_value.putheader.assert_has_calls(calls, False)
        mock_poc.return_value.endheaders.assert_called_once_with()

        # The following steps are only taken if the initial status is 100
        if initial_status == 100:
            # Verify that the method returned what it was supposed to
            self.assertEqual(status, 201)

            # Verify that _safe_read was called once to remove the CRLF
            # after the 100 response
            mock_rc = mock_poc.return_value.response_class.return_value
            mock_rc._safe_read.assert_called_once_with(2)

            # Verify the actual data was written via send
            mock_poc.return_value.send.assert_called_once_with(req_data)

            # Verify that the getresponse method was called to receive
            # the final
            mock_poc.return_value.getresponse.assert_called_once_with()



# Copyright (C) 2017 Dell Inc. or its subsidiaries.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from tempest.lib.services.volume.v3 import group_types_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestGroupTypesClient(base.BaseServiceTest):
    FAKE_CREATE_GROUP_TYPE = {
        "group_type": {
            "id": "6685584b-1eac-4da6-b5c3-555430cf68ff",
            "name": "group-type-001",
            "description": "Test group type 1",
            "is_public": True,
            "group_specs": {
                "consistent_group_snapshot_enabled": "<is> False"
            }
        }
    }

    FAKE_INFO_GROUP_TYPE = {
        "group_type": {
            "id": "0e701ab8-1bec-4b9f-b026-a7ba4af13578",
            "name": "group-type-001",
            "description": "Test group type 1",
            "is_public": True,
            "group_specs": {},
        }
    }

    FAKE_INFO_DEFAULT_GROUP_TYPE = {
        "group_type": {
            "id": "7270c56e-6354-4528-8e8b-f54dee2232c8",
            "name": "group-type-default",
            "description": "default group type",
            "is_public": True,
            "group_specs": {},
        }
    }

    FAKE_LIST_GROUP_TYPES = {
        "group_types": [
            {
                "id": "0e701ab8-1bec-4b9f-b026-a7ba4af13578",
                "name": "group-type-001",
                "description": "Test group type 1",
                "is_public": True,
                "group_specs": {
                    "consistent_group_snapshot_enabled": "<is> False"
                }
            },
            {
                "id": "e479997c-650b-40a4-9dfe-77655818b0d2",
                "name": "group-type-002",
                "description": "Test group type 2",
                "is_public": True,
                "group_specs": {
                    "consistent_group_snapshot_enabled": "<is> False"
                }
            },
            {
                "id": "c5c4769e-213c-40a6-a568-8e797bb691d4",
                "name": "group-type-003",
                "description": "Test group type 3",
                "is_public": True,
                "group_specs": {
                    "consistent_group_snapshot_enabled": "<is> False"
                }
            }
        ]
    }

    FAKE_CREATE_GROUP_TYPE_SPECS = {
        "group_specs": {
            "key1": "value1",
            "key2": "value2"
        }
    }

    FAKE_LIST_GROUP_TYPE_SPECS = {
        "group_specs": {
            "key1": "value1",
            "key2": "value2"
        }
    }

    FAKE_SHOW_GROUP_TYPE_SPECS_ITEM = {
        "key1": "value1"
    }

    FAKE_UPDATE_GROUP_TYPE_SPECS_ITEM = {
        "key2": "value2-updated"
    }

    def setUp(self):
        super(TestGroupTypesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = group_types_client.GroupTypesClient(fake_auth,
                                                          'volume',
                                                          'regionOne')

    def _test_create_group_type(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_group_type,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_GROUP_TYPE,
            bytes_body,
            status=202)

    def _test_show_group_type(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_group_type,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_INFO_GROUP_TYPE,
            bytes_body,
            group_type_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_show_default_group_type(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_default_group_type,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_INFO_DEFAULT_GROUP_TYPE,
            bytes_body)

    def _test_list_group_types(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_group_types,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_GROUP_TYPES,
            bytes_body)

    def _test_update_group_types(self, bytes_body=False):
        resp_body = copy.deepcopy(self.FAKE_INFO_GROUP_TYPE)
        self.check_service_client_function(
            self.client.update_group_type,
            'tempest.lib.common.rest_client.RestClient.put',
            resp_body,
            bytes_body,
            group_type_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_create_or_update_group_type_specs(self, bytes_body=False):
        group_specs = self.FAKE_CREATE_GROUP_TYPE_SPECS['group_specs']
        self.check_service_client_function(
            self.client.create_or_update_group_type_specs,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_GROUP_TYPE_SPECS,
            bytes_body,
            group_type_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5",
            group_specs=group_specs,
            status=202)

    def _test_list_group_type_specs(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_group_type_specs,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_GROUP_TYPE_SPECS,
            bytes_body,
            group_type_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_show_group_type_specs_item(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_group_type_specs_item,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_GROUP_TYPE_SPECS_ITEM,
            bytes_body,
            group_type_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5",
            spec_id="key1")

    def _test_update_group_type_specs_item(self, bytes_body=False):
        spec = self.FAKE_UPDATE_GROUP_TYPE_SPECS_ITEM
        self.check_service_client_function(
            self.client.update_group_type_specs_item,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_GROUP_TYPE_SPECS_ITEM,
            bytes_body,
            group_type_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5",
            spec_id="key2",
            spec=spec)

    def test_create_group_type_with_str_body(self):
        self._test_create_group_type()

    def test_create_group_type_with_bytes_body(self):
        self._test_create_group_type(bytes_body=True)

    def test_delete_group_type(self):
        self.check_service_client_function(
            self.client.delete_group_type,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            group_type_id='0e58433f-d108-4bf3-a22c-34e6b71ef86b',
            status=202)

    def test_show_group_type_with_str_body(self):
        self._test_show_group_type()

    def test_show_group_type_with_bytes_body(self):
        self._test_show_group_type(bytes_body=True)

    def test_show_default_group_type_with_str_body(self):
        self._test_show_default_group_type()

    def test_show_default_group_type_with_bytes_body(self):
        self._test_show_default_group_type(bytes_body=True)

    def test_list_group_types_with_str_body(self):
        self._test_list_group_types()

    def test_list_group_types_with_bytes_body(self):
        self._test_list_group_types(bytes_body=True)

    def test_update_group_types_with_str_body(self):
        self._test_update_group_types()

    def test_update_group_types_with_bytes_body(self):
        self._test_update_group_types(bytes_body=True)

    def test_create_or_update_group_type_specs_with_str_body(self):
        self._test_create_or_update_group_type_specs()

    def test_create_or_update_group_type_specs_with_bytes_body(self):
        self._test_create_or_update_group_type_specs(bytes_body=True)

    def test_list_group_type_specs_with_str_body(self):
        self._test_list_group_type_specs()

    def test_list_group_type_specs_with_bytes_body(self):
        self._test_list_group_type_specs(bytes_body=True)

    def test_show_group_type_specs_item_with_str_body(self):
        self._test_show_group_type_specs_item()

    def test_show_group_type_specs_item_with_bytes_body(self):
        self._test_show_group_type_specs_item(bytes_body=True)

    def test_update_group_type_specs_item_with_str_body(self):
        self._test_update_group_type_specs_item()

    def test_update_group_type_specs_item_with_bytes_body(self):
        self._test_update_group_type_specs_item(bytes_body=True)

    def test_delete_group_type_specs_item(self):
        self.check_service_client_function(
            self.client.delete_group_type_specs_item,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            group_type_id='0e58433f-d108-4bf3-a22c-34e6b71ef86b',
            spec_id='key1',
            status=202)



# Copyright 2013 IBM Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import fixed_ips as schema
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class FixedIPsClient(base_compute_client.BaseComputeClient):

    def show_fixed_ip(self, fixed_ip):
        url = "os-fixed-ips/%s" % fixed_ip
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.get_fixed_ip, resp, body)
        return rest_client.ResponseBody(resp, body)

    def reserve_fixed_ip(self, fixed_ip, **kwargs):
        """Reserve/Unreserve a fixed IP.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#reserve-or-release-a-fixed-ip
        """
        url = "os-fixed-ips/%s/action" % fixed_ip
        resp, body = self.post(url, json.dumps(kwargs))
        self.validate_response(schema.reserve_unreserve_fixed_ip, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import agents as schema
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class AgentsClient(base_compute_client.BaseComputeClient):
    """Tests Agents API"""

    def list_agents(self, **params):
        """List all agent builds.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-agent-builds
        """
        url = 'os-agents'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_agents, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_agent(self, **kwargs):
        """Create an agent build.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-agent-build
        """
        post_body = json.dumps({'agent': kwargs})
        resp, body = self.post('os-agents', post_body)
        body = json.loads(body)
        self.validate_response(schema.create_agent, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_agent(self, agent_id):
        """Delete an existing agent build.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-agent-build
        """
        resp, body = self.delete("os-agents/%s" % agent_id)
        self.validate_response(schema.delete_agent, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_agent(self, agent_id, **kwargs):
        """Update an agent build.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-agent-build
        """
        put_body = json.dumps({'para': kwargs})
        resp, body = self.put('os-agents/%s' % agent_id, put_body)
        body = json.loads(body)
        self.validate_response(schema.update_agent, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import subnets_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSubnetsClient(base.BaseServiceTest):

    FAKE_SUBNET = {
        "subnet": {
            "name": "",
            "enable_dhcp": True,
            "network_id": "d32019d3-bc6e-4319-9c1d-6722fc136a22",
            "segment_id": None,
            "project_id": "4fd44f30292945e481c7b8a0c8908869",
            "tenant_id": "4fd44f30292945e481c7b8a0c8908869",
            "dns_nameservers": [],
            "allocation_pools": [
                {
                    "start": "192.168.199.2",
                    "end": "192.168.199.254"
                }
            ],
            "host_routes": [],
            "ip_version": 4,
            "gateway_ip": "192.168.199.1",
            "cidr": "192.168.199.0/24",
            "id": "3b80198d-4f7b-4f77-9ef5-774d54e17126",
            "created_at": "2016-10-10T14:35:47Z",
            "description": "",
            "ipv6_address_mode": None,
            "ipv6_ra_mode": None,
            "revision_number": 2,
            "service_types": [],
            "subnetpool_id": None,
            "updated_at": "2016-10-10T14:35:47Z"
        }
    }

    FAKE_UPDATED_SUBNET = {
        "subnet": {
            "name": "my_subnet",
            "enable_dhcp": True,
            "network_id": "db193ab3-96e3-4cb3-8fc5-05f4296d0324",
            "segment_id": None,
            "project_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
            "tenant_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
            "dns_nameservers": [],
            "allocation_pools": [
                {
                    "start": "10.0.0.2",
                    "end": "10.0.0.254"
                }
            ],
            "host_routes": [],
            "ip_version": 4,
            "gateway_ip": "10.0.0.1",
            "cidr": "10.0.0.0/24",
            "id": "08eae331-0402-425a-923c-34f7cfe39c1b",
            "description": ""
        }
    }

    FAKE_SUBNETS = {
        "subnets": [
            {
                "name": "private-subnet",
                "enable_dhcp": True,
                "network_id": "db193ab3-96e3-4cb3-8fc5-05f4296d0324",
                "segment_id": None,
                "project_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "tenant_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "dns_nameservers": [],
                "allocation_pools": [
                    {
                        "start": "10.0.0.2",
                        "end": "10.0.0.254"
                    }
                ],
                "host_routes": [],
                "ip_version": 4,
                "gateway_ip": "10.0.0.1",
                "cidr": "10.0.0.0/24",
                "id": "08eae331-0402-425a-923c-34f7cfe39c1b",
                "created_at": "2016-10-10T14:35:34Z",
                "description": "",
                "ipv6_address_mode": None,
                "ipv6_ra_mode": None,
                "revision_number": 2,
                "service_types": [],
                "subnetpool_id": None,
                "updated_at": "2016-10-10T14:35:34Z"
            },
            {
                "name": "my_subnet",
                "enable_dhcp": True,
                "network_id": "d32019d3-bc6e-4319-9c1d-6722fc136a22",
                "segment_id": None,
                "project_id": "4fd44f30292945e481c7b8a0c8908869",
                "tenant_id": "4fd44f30292945e481c7b8a0c8908869",
                "dns_nameservers": [],
                "allocation_pools": [
                    {
                        "start": "192.0.0.2",
                        "end": "192.255.255.254"
                    }
                ],
                "host_routes": [],
                "ip_version": 4,
                "gateway_ip": "192.0.0.1",
                "cidr": "192.0.0.0/8",
                "id": "54d6f61d-db07-451c-9ab3-b9609b6b6f0b",
                "created_at": "2016-10-10T14:35:47Z",
                "description": "",
                "ipv6_address_mode": None,
                "ipv6_ra_mode": None,
                "revision_number": 2,
                "service_types": [],
                "subnetpool_id": None,
                "updated_at": "2016-10-10T14:35:47Z"
            }
        ]
    }

    FAKE_BULK_SUBNETS = copy.deepcopy(FAKE_SUBNETS)

    FAKE_SUBNET_ID = "54d6f61d-db07-451c-9ab3-b9609b6b6f0b"

    FAKE_NETWORK_ID = "d32019d3-bc6e-4319-9c1d-6722fc136a22"

    def setUp(self):
        super(TestSubnetsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.subnets_client = subnets_client.SubnetsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_create_subnet(self, bytes_body=False):
        self.check_service_client_function(
            self.subnets_client.create_subnet,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SUBNET,
            bytes_body,
            201,
            network_id=self.FAKE_NETWORK_ID,
            ip_version=4,
            cidr="192.168.199.0/24")

    def _test_update_subnet(self, bytes_body=False):
        self.check_service_client_function(
            self.subnets_client.update_subnet,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATED_SUBNET,
            bytes_body,
            200,
            subnet_id=self.FAKE_SUBNET_ID,
            name="fake_updated_subnet_name")

    def _test_show_subnet(self, bytes_body=False):
        self.check_service_client_function(
            self.subnets_client.show_subnet,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SUBNET,
            bytes_body,
            200,
            subnet_id=self.FAKE_SUBNET_ID)

    def _test_list_subnets(self, bytes_body=False):
        self.check_service_client_function(
            self.subnets_client.list_subnets,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SUBNETS,
            bytes_body,
            200)

    def _test_create_bulk_subnets(self, bytes_body=False):
        kwargs = {
            "subnets": [
                {
                    "cidr": "192.168.199.0/24",
                    "ip_version": 4,
                    "network_id": "e6031bc2-901a-4c66-82da-f4c32ed89406"
                },
                {
                    "cidr": "10.56.4.0/22",
                    "ip_version": 4,
                    "network_id": "64239a54-dcc4-4b39-920b-b37c2144effa"
                }
            ]
        }
        self.check_service_client_function(
            self.subnets_client.create_bulk_subnets,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SUBNETS,
            bytes_body,
            201,
            **kwargs)

    def test_create_subnet_with_str_body(self):
        self._test_create_subnet()

    def test_create_subnet_with_bytes_body(self):
        self._test_create_subnet(bytes_body=True)

    def test_update_subnet_with_str_body(self):
        self._test_update_subnet()

    def test_update_subnet_with_bytes_body(self):
        self._test_update_subnet(bytes_body=True)

    def test_show_subnet_with_str_body(self):
        self._test_show_subnet()

    def test_show_subnet_with_bytes_body(self):
        self._test_show_subnet(bytes_body=True)

    def test_list_subnets_with_str_body(self):
        self._test_list_subnets()

    def test_list_subnets_with_bytes_body(self):
        self._test_list_subnets(bytes_body=True)

    def test_delete_subnet(self):
        self.check_service_client_function(
            self.subnets_client.delete_subnet,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            subnet_id=self.FAKE_SUBNET_ID)

    def test_create_bulk_subnets_with_str_body(self):
        self._test_create_bulk_subnets()

    def test_create_bulk_subnets_with_bytes_body(self):
        self._test_create_bulk_subnets(bytes_body=True)



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common import api_version_utils
from tempest.lib.common import rest_client

VOLUME_MICROVERSION = None


class BaseClient(rest_client.RestClient):
    """Base volume service clients class to support microversion."""
    api_microversion_header_name = 'Openstack-Api-Version'

    def get_headers(self, accept_type=None, send_type=None):
        headers = super(BaseClient, self).get_headers(
            accept_type=accept_type, send_type=send_type)
        if VOLUME_MICROVERSION:
            headers[self.api_microversion_header_name] = ('volume %s' %
                                                          VOLUME_MICROVERSION)
        return headers

    def request(self, method, url, extra_headers=False, headers=None,
                body=None, chunked=False):

        resp, resp_body = super(BaseClient, self).request(
            method, url, extra_headers, headers, body, chunked)
        if (VOLUME_MICROVERSION and
            VOLUME_MICROVERSION != api_version_utils.LATEST_MICROVERSION):
            api_version_utils.assert_version_header_matches_request(
                self.api_microversion_header_name,
                'volume %s' % VOLUME_MICROVERSION,
                resp)
        return resp, resp_body



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import auth


class FakeCredentials(auth.Credentials):

    def is_valid(self):
        return True


class FakeKeystoneV2Credentials(auth.KeystoneV2Credentials):

    def __init__(self):
        creds = dict(
            username='fake_username',
            password='fake_password',
            tenant_name='fake_tenant_name'
        )
        super(FakeKeystoneV2Credentials, self).__init__(**creds)


class FakeKeystoneV3Credentials(auth.KeystoneV3Credentials):
    """Fake credentials suitable for the Keystone Identity V3 API"""

    def __init__(self):
        creds = dict(
            username='fake_username',
            password='fake_password',
            user_domain_name='fake_domain_name',
            project_name='fake_tenant_name',
            project_domain_name='fake_domain_name'
        )
        super(FakeKeystoneV3Credentials, self).__init__(**creds)


class FakeKeystoneV3DomainCredentials(auth.KeystoneV3Credentials):
    """Fake credentials for the Keystone Identity V3 API, with no scope"""

    def __init__(self):
        creds = dict(
            username='fake_username',
            password='fake_password',
            user_domain_name='fake_domain_name'
        )
        super(FakeKeystoneV3DomainCredentials, self).__init__(**creds)


class FakeKeystoneV3AllCredentials(auth.KeystoneV3Credentials):
    """Fake credentials for the Keystone Identity V3 API, with no scope"""

    def __init__(self):
        creds = dict(
            username='fake_username',
            password='fake_password',
            user_domain_name='fake_domain_name',
            project_name='fake_tenant_name',
            project_domain_name='fake_domain_name',
            domain_name='fake_domain_name'
        )
        super(FakeKeystoneV3AllCredentials, self).__init__(**creds)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions
from tempest.lib.services.identity.v3 import token_client
from tempest.tests import base
from tempest.tests.lib import fake_identity


class TestTokenClientV3(base.TestCase):

    def test_init_without_authurl(self):
        self.assertRaises(exceptions.IdentityError,
                          token_client.V3TokenClient, None)

    def test_auth(self):
        token_client_v3 = token_client.V3TokenClient('fake_url')
        response, body_text = fake_identity._fake_v3_response(None, None)
        body = json.loads(body_text)

        with mock.patch.object(token_client_v3, 'post') as post_mock:
            post_mock.return_value = response, body
            resp = token_client_v3.auth(username='fake_user',
                                        password='fake_pass')

        self.assertIsInstance(resp, rest_client.ResponseBody)
        req_dict = json.dumps({
            'auth': {
                'identity': {
                    'methods': ['password'],
                    'password': {
                        'user': {
                            'name': 'fake_user',
                            'password': 'fake_pass',
                        }
                    }
                },
            }
        }, sort_keys=True)
        post_mock.assert_called_once_with('fake_url/auth/tokens',
                                          body=req_dict)

    def test_auth_with_project_id_and_domain_id(self):
        token_client_v3 = token_client.V3TokenClient('fake_url')
        response, body_text = fake_identity._fake_v3_response(None, None)
        body = json.loads(body_text)

        with mock.patch.object(token_client_v3, 'post') as post_mock:
            post_mock.return_value = response, body
            resp = token_client_v3.auth(
                username='fake_user', password='fake_pass',
                project_id='fcac2a055a294e4c82d0a9c21c620eb4',
                user_domain_id='14f4a9a99973404d8c20ba1d2af163ff',
                project_domain_id='291f63ae9ac54ee292ca09e5f72d9676')

        self.assertIsInstance(resp, rest_client.ResponseBody)
        req_dict = json.dumps({
            'auth': {
                'identity': {
                    'methods': ['password'],
                    'password': {
                        'user': {
                            'name': 'fake_user',
                            'password': 'fake_pass',
                            'domain': {
                                'id': '14f4a9a99973404d8c20ba1d2af163ff'
                            }
                        }
                    }
                },
                'scope': {
                    'project': {
                        'id': 'fcac2a055a294e4c82d0a9c21c620eb4',
                        'domain': {
                            'id': '291f63ae9ac54ee292ca09e5f72d9676'
                        }
                    }
                }
            }
        }, sort_keys=True)
        post_mock.assert_called_once_with('fake_url/auth/tokens',
                                          body=req_dict)

    def test_auth_with_tenant(self):
        token_client_v3 = token_client.V3TokenClient('fake_url')
        response, body_text = fake_identity._fake_v3_response(None, None)
        body = json.loads(body_text)

        with mock.patch.object(token_client_v3, 'post') as post_mock:
            post_mock.return_value = response, body
            resp = token_client_v3.auth(username='fake_user',
                                        password='fake_pass',
                                        project_name='fake_tenant')

        self.assertIsInstance(resp, rest_client.ResponseBody)
        req_dict = json.dumps({
            'auth': {
                'identity': {
                    'methods': ['password'],
                    'password': {
                        'user': {
                            'name': 'fake_user',
                            'password': 'fake_pass',
                        }
                    }},
                'scope': {
                    'project': {
                        'name': 'fake_tenant'
                    }
                },
            }
        }, sort_keys=True)

        post_mock.assert_called_once_with('fake_url/auth/tokens',
                                          body=req_dict)

    def test_request_with_str_body(self):
        token_client_v3 = token_client.V3TokenClient('fake_url')

        with mock.patch.object(token_client_v3, 'raw_request') as mock_raw_r:
            mock_raw_r.return_value = (
                fake_identity._fake_v3_response(None, None))
            resp, body = token_client_v3.request('GET', 'fake_uri')
        mock_raw_r.assert_called_once_with('fake_uri', 'GET',
                                           headers=mock.ANY, body=None,
                                           log_req_body='<omitted>')

        self.assertIsInstance(body, dict)

    def test_request_with_bytes_body(self):
        token_client_v3 = token_client.V3TokenClient('fake_url')

        response, body_text = fake_identity._fake_v3_response(None, None)
        body = body_text.encode('utf-8')

        with mock.patch.object(token_client_v3, 'raw_request') as mock_raw_r:
            mock_raw_r.return_value = response, body
            resp, body = token_client_v3.request('GET', 'fake_uri')

        self.assertIsInstance(body, dict)



# Copyright 2013 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import tempest_fixtures as fixtures
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class AggregatesAdminTestBase(base.BaseV2ComputeAdminTest):
    """Tests Aggregates API that require admin privileges"""

    @classmethod
    def setup_clients(cls):
        super(AggregatesAdminTestBase, cls).setup_clients()
        cls.client = cls.os_admin.aggregates_client

    @classmethod
    def resource_setup(cls):
        super(AggregatesAdminTestBase, cls).resource_setup()
        cls.aggregate_name_prefix = 'test_aggregate'
        cls.az_name_prefix = 'test_az'

        cls.host = None
        hypers = cls.os_admin.hypervisor_client.list_hypervisors(
            detail=True)['hypervisors']

        if CONF.compute.hypervisor_type:
            hypers = [hyper for hyper in hypers
                      if (hyper['hypervisor_type'] ==
                          CONF.compute.hypervisor_type)]

        cls.hosts_available = [hyper['service']['host'] for hyper in hypers
                               if (hyper['state'] == 'up' and
                                   hyper['status'] == 'enabled')]
        if cls.hosts_available:
            cls.host = cls.hosts_available[0]
        else:
            msg = "no available compute node found"
            if CONF.compute.hypervisor_type:
                msg += " for hypervisor_type %s" % CONF.compute.hypervisor_type
            raise testtools.TestCase.failureException(msg)

    def _create_test_aggregate(self, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name(self.aggregate_name_prefix)
        aggregate = self.client.create_aggregate(**kwargs)['aggregate']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.client.delete_aggregate, aggregate['id'])
        self.assertEqual(kwargs['name'], aggregate['name'])

        return aggregate


class AggregatesAdminTestJSON(AggregatesAdminTestBase):
    """Tests Aggregates API that require admin privileges"""

    @decorators.idempotent_id('0d148aa3-d54c-4317-aa8d-42040a475e20')
    def test_aggregate_create_delete(self):
        """Test create/delete aggregate"""
        aggregate = self._create_test_aggregate()
        self.assertIsNone(aggregate['availability_zone'])

        self.client.delete_aggregate(aggregate['id'])
        self.client.wait_for_resource_deletion(aggregate['id'])

    @decorators.idempotent_id('5873a6f8-671a-43ff-8838-7ce430bb6d0b')
    def test_aggregate_create_delete_with_az(self):
        """Test create/delete aggregate with availability_zone"""
        az_name = data_utils.rand_name(self.az_name_prefix)
        aggregate = self._create_test_aggregate(availability_zone=az_name)
        self.assertEqual(az_name, aggregate['availability_zone'])

        self.client.delete_aggregate(aggregate['id'])
        self.client.wait_for_resource_deletion(aggregate['id'])

    @decorators.idempotent_id('68089c38-04b1-4758-bdf0-cf0daec4defd')
    def test_aggregate_create_verify_entry_in_list(self):
        """Test listing aggregate should contain the created aggregate"""
        aggregate = self._create_test_aggregate()
        aggregates = self.client.list_aggregates()['aggregates']
        self.assertIn((aggregate['id'], aggregate['availability_zone']),
                      map(lambda x: (x['id'], x['availability_zone']),
                          aggregates))

    @decorators.idempotent_id('36ec92ca-7a73-43bc-b920-7531809e8540')
    def test_aggregate_create_update_metadata_get_details(self):
        """Test set/get aggregate metadata"""
        aggregate = self._create_test_aggregate()
        body = self.client.show_aggregate(aggregate['id'])['aggregate']
        self.assertEqual(aggregate['name'], body['name'])
        self.assertEqual(aggregate['availability_zone'],
                         body['availability_zone'])
        self.assertEqual({}, body["metadata"])

        # set the metadata of the aggregate
        meta = {"key": "value"}
        body = self.client.set_metadata(aggregate['id'], metadata=meta)
        self.assertEqual(meta, body['aggregate']["metadata"])

        # verify the metadata has been set
        body = self.client.show_aggregate(aggregate['id'])['aggregate']
        self.assertEqual(meta, body["metadata"])

    @decorators.idempotent_id('4d2b2004-40fa-40a1-aab2-66f4dab81beb')
    def test_aggregate_create_update_with_az(self):
        """Test create/update aggregate with availability_zone"""
        aggregate_name = data_utils.rand_name(self.aggregate_name_prefix)
        az_name = data_utils.rand_name(self.az_name_prefix)
        aggregate = self._create_test_aggregate(
            name=aggregate_name, availability_zone=az_name)

        self.assertEqual(az_name, aggregate['availability_zone'])

        aggregate_id = aggregate['id']
        new_aggregate_name = aggregate_name + '_new'
        new_az_name = az_name + '_new'

        resp_aggregate = self.client.update_aggregate(
            aggregate_id,
            name=new_aggregate_name,
            availability_zone=new_az_name)['aggregate']
        self.assertEqual(new_aggregate_name, resp_aggregate['name'])
        self.assertEqual(new_az_name, resp_aggregate['availability_zone'])

        aggregates = self.client.list_aggregates()['aggregates']
        self.assertIn((aggregate_id, new_aggregate_name, new_az_name),
                      map(lambda x:
                          (x['id'], x['name'], x['availability_zone']),
                          aggregates))

    @decorators.idempotent_id('c8e85064-e79b-4906-9931-c11c24294d02')
    def test_aggregate_add_remove_host(self):
        """Test adding host to and removing host from aggregate"""
        self.useFixture(fixtures.LockFixture('availability_zone'))
        aggregate_name = data_utils.rand_name(self.aggregate_name_prefix)
        aggregate = self._create_test_aggregate(name=aggregate_name)

        body = (self.client.add_host(aggregate['id'], host=self.host)
                ['aggregate'])
        self.assertEqual(aggregate_name, body['name'])
        self.assertEqual(aggregate['availability_zone'],
                         body['availability_zone'])
        self.assertIn(self.host, body['hosts'])

        body = (self.client.remove_host(aggregate['id'], host=self.host)
                ['aggregate'])
        self.assertEqual(aggregate_name, body['name'])
        self.assertEqual(aggregate['availability_zone'],
                         body['availability_zone'])
        self.assertNotIn(self.host, body['hosts'])

    @decorators.idempotent_id('7f6a1cc5-2446-4cdb-9baa-b6ae0a919b72')
    def test_aggregate_add_host_list(self):
        """Test listing aggregate contains the host added to the aggregate

        Add a host to the given aggregate and list.
        """
        self.useFixture(fixtures.LockFixture('availability_zone'))
        aggregate_name = data_utils.rand_name(self.aggregate_name_prefix)
        aggregate = self._create_test_aggregate(name=aggregate_name)

        self.client.add_host(aggregate['id'], host=self.host)
        self.addCleanup(self.client.remove_host, aggregate['id'],
                        host=self.host)

        aggregates = self.client.list_aggregates()['aggregates']
        aggs = [agg for agg in aggregates if agg['id'] == aggregate['id']]
        self.assertEqual(1, len(aggs))
        agg = aggs[0]
        self.assertEqual(aggregate_name, agg['name'])
        self.assertIsNone(agg['availability_zone'])
        self.assertIn(self.host, agg['hosts'])

    @decorators.idempotent_id('eeef473c-7c52-494d-9f09-2ed7fc8fc036')
    def test_aggregate_add_host_get_details(self):
        """Test showing aggregate contains the host added to the aggregate

        Add a host to the given aggregate and get details.
        """
        self.useFixture(fixtures.LockFixture('availability_zone'))
        aggregate_name = data_utils.rand_name(self.aggregate_name_prefix)
        aggregate = self._create_test_aggregate(name=aggregate_name)

        self.client.add_host(aggregate['id'], host=self.host)
        self.addCleanup(self.client.remove_host, aggregate['id'],
                        host=self.host)

        body = self.client.show_aggregate(aggregate['id'])['aggregate']
        self.assertEqual(aggregate_name, body['name'])
        self.assertIsNone(body['availability_zone'])
        self.assertIn(self.host, body['hosts'])

    @decorators.idempotent_id('96be03c7-570d-409c-90f8-e4db3c646996')
    def test_aggregate_add_host_create_server_with_az(self):
        """Test adding a host to the given aggregate and creating a server"""
        self.useFixture(fixtures.LockFixture('availability_zone'))
        az_name = data_utils.rand_name(self.az_name_prefix)
        aggregate = self._create_test_aggregate(availability_zone=az_name)

        # Find a host that has not been added to other availability zone,
        # for one host can't be added to different availability zones.
        aggregates = self.client.list_aggregates()['aggregates']
        hosts_in_zone = []
        for agg in aggregates:
            if agg['availability_zone']:
                hosts_in_zone.extend(agg['hosts'])
        hosts = [v for v in self.hosts_available if v not in hosts_in_zone]
        if not hosts:
            raise self.skipException("All hosts are already in other "
                                     "availability zones, so can't add "
                                     "host to aggregate. \nAggregates list: "
                                     "%s" % aggregates)
        host = hosts[0]

        self.client.add_host(aggregate['id'], host=host)
        self.addCleanup(self.client.remove_host, aggregate['id'], host=host)
        server = self.create_test_server(availability_zone=az_name,
                                         wait_until='ACTIVE')
        server_host = self.get_host_for_server(server['id'])
        self.assertEqual(host, server_host)


class AggregatesAdminTestV241(AggregatesAdminTestBase):
    """Tests Aggregates API that require admin privileges

    Tests Aggregates API that require admin privileges with compute
    microversion greater than 2.40.
    """
    min_microversion = '2.41'

    # NOTE(gmann): This test tests the Aggregate APIs response schema
    # for 2.41 microversion. No specific assert or behaviour verification
    # is needed.

    @decorators.idempotent_id('fdf24d9e-8afa-4700-b6aa-9c498351504f')
    def test_create_update_show_aggregate_add_remove_host(self):
        """Test response schema of aggregates API

        Test response schema of aggregates API(create/update/show/add host/
        remove host) with compute microversion greater than 2.40.
        """
        # Update and add a host to the given aggregate and get details.
        self.useFixture(fixtures.LockFixture('availability_zone'))
        # Checking create aggregate API response schema
        aggregate = self._create_test_aggregate()

        new_aggregate_name = data_utils.rand_name(self.aggregate_name_prefix)
        # Checking update aggregate API response schema
        self.client.update_aggregate(aggregate['id'], name=new_aggregate_name)
        # Checking show aggregate API response schema
        self.client.show_aggregate(aggregate['id'])['aggregate']
        # Checking add host to aggregate API response schema
        self.client.add_host(aggregate['id'], host=self.host)
        # Checking rempve host from aggregate API response schema
        self.client.remove_host(aggregate['id'], host=self.host)



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class GroupTypeSpecsTest(base.BaseVolumeAdminTest):
    """Test group type specs"""

    min_microversion = '3.11'
    max_microversion = 'latest'

    @decorators.idempotent_id('bb4e30d0-de6e-4f4d-866c-dcc48d023b4e')
    def test_group_type_specs_create_show_update_list_delete(self):
        """Test create/show/update/list/delete group type specs"""
        # Create new group type
        group_type = self.create_group_type()

        # Create new group type specs
        create_specs = {
            "key1": "value1",
            "key2": "value2"
        }
        body = self.admin_group_types_client.create_or_update_group_type_specs(
            group_type['id'], create_specs)['group_specs']
        self.assertEqual(create_specs, body)

        # Create a new group type spec and update an existing group type spec
        update_specs = {
            "key2": "value2-updated",
            "key3": "value3"
        }
        body = self.admin_group_types_client.create_or_update_group_type_specs(
            group_type['id'], update_specs)['group_specs']
        self.assertEqual(update_specs, body)

        # Show specified item of group type specs
        spec_keys = ['key2', 'key3']
        for key in spec_keys:
            body = self.admin_group_types_client.show_group_type_specs_item(
                group_type['id'], key)
            self.assertIn(key, body)
            self.assertEqual(update_specs[key], body[key])

        # Update specified item of group type specs
        update_key = 'key3'
        update_spec = {update_key: "value3-updated"}
        body = self.admin_group_types_client.update_group_type_specs_item(
            group_type['id'], update_key, update_spec)
        self.assertEqual(update_spec, body)

        # List all group type specs that created or updated above
        list_specs = {}
        list_specs.update(create_specs)
        list_specs.update(update_specs)
        list_specs.update(update_spec)
        body = self.admin_group_types_client.list_group_type_specs(
            group_type['id'])['group_specs']
        self.assertEqual(list_specs, body)

        # Delete specified item of group type specs
        delete_key = 'key1'
        self.admin_group_types_client.delete_group_type_specs_item(
            group_type['id'], delete_key)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_group_types_client.show_group_type_specs_item,
            group_type['id'], delete_key)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

manage_snapshot = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshot': {
                'type': 'object',
                'properties': {
                    'status': {'type': 'string'},
                    'size': {'type': 'integer'},
                    'metadata': {
                        'type': 'object',
                        'patternProperties': {
                            '^.+$': {'type': 'string'}
                        }
                    },
                    'name': {'type': ['string', 'null']},
                    'volume_id': {'type': 'string', 'format': 'uuid'},
                    'created_at': parameter_types.date_time,
                    'description': {'type': ['string', 'null']},
                    'id': {'type': 'string', 'format': 'uuid'},
                    'updated_at': parameter_types.date_time_or_null
                },
                'additionalProperties': False,
                'required': ['status', 'size', 'volume_id',
                             'created_at', 'description', 'id', 'updated_at']
            }
        },
        'additionalProperties': False,
        'required': ['snapshot']
    }
}



# Copyright 2016 EasyStack.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.image.v2 import namespace_properties_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestNamespacePropertiesClient(base.BaseServiceTest):
    FAKE_CREATE_SHOW_NAMESPACE_PROPERTY = {
        "description": "property",
        "enum": ["xen", "qemu", "kvm", "lxc", "uml", "vmware", "hyperv"],
        "name": "OS::Glance::Image",
        "title": "Hypervisor Type",
        "type": "string"
    }

    FAKE_LIST_NAMESPACE_PROPERTY = {
        "properties": {
            "hw_disk_bus": {
                "description": "property.",
                "enum": ["scsi", "virtio", "uml", "xen", "ide", "usb"],
                "title": "Disk Bus",
                "type": "string"
            },
            "hw_machine_type": {
                "description": "desc.",
                "title": "Machine Type",
                "type": "string"
            },
            "hw_qemu_guest_agent": {
                "description": "desc.",
                "enum": [
                    "yes",
                    "no"
                ],
                "title": "QEMU Guest Agent",
                "type": "string"
            },
            "hw_rng_model": {
                "default": "virtio",
                "description": "desc",
                "title": "Random Number Generator Device",
                "type": "string"
            },
            "hw_scsi_model": {
                "default": "virtio-scsi",
                "description": "desc.",
                "title": "SCSI Model",
                "type": "string"
            },
            "hw_video_model": {
                "description": "The video image driver used.",
                "enum": [
                    "vga",
                    "cirrus",
                    "vmvga",
                    "xen",
                    "qxl"
                ],
                "title": "Video Model",
                "type": "string"
            },
            "hw_video_ram": {
                "description": "desc.",
                "title": "Max Video Ram",
                "type": "integer"
            },
            "hw_vif_model": {
                "description": "desc.",
                "enum": ["e1000",
                         "ne2k_pci",
                         "pcnet",
                         "rtl8139",
                         "virtio",
                         "e1000",
                         "e1000e",
                         "VirtualE1000",
                         "VirtualE1000e",
                         "VirtualPCNet32",
                         "VirtualSriovEthernetCard",
                         "VirtualVmxnet",
                         "netfront",
                         "ne2k_pci"
                         ],
                "title": "Virtual Network Interface",
                "type": "string"
            },
            "os_command_line": {
                "description": "desc.",
                "title": "Kernel Command Line",
                "type": "string"
            }
        }
    }

    FAKE_UPDATE_NAMESPACE_PROPERTY = {
        "description": "property",
        "enum": ["xen", "qemu", "kvm", "lxc", "uml", "vmware", "hyperv"],
        "name": "OS::Glance::Image",
        "title": "update Hypervisor Type",
        "type": "string"
    }

    def setUp(self):
        super(TestNamespacePropertiesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = namespace_properties_client.NamespacePropertiesClient(
            fake_auth, 'image', 'regionOne')

    def _test_create_namespace_property(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_namespace_property,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_SHOW_NAMESPACE_PROPERTY,
            bytes_body, status=201,
            namespace="OS::Compute::Hypervisor",
            title="Hypervisor Type", name="OS::Glance::Image",
            type="string",
            enum=["xen", "qemu", "kvm", "lxc", "uml", "vmware", "hyperv"])

    def _test_list_namespace_property(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_namespace_properties,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_NAMESPACE_PROPERTY,
            bytes_body,
            namespace="OS::Compute::Hypervisor")

    def _test_show_namespace_property(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_namespace_properties,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CREATE_SHOW_NAMESPACE_PROPERTY,
            bytes_body,
            namespace="OS::Compute::Hypervisor",
            property_name="OS::Glance::Image")

    def _test_update_namespace_property(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_namespace_properties,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_NAMESPACE_PROPERTY,
            bytes_body,
            namespace="OS::Compute::Hypervisor",
            property_name="OS::Glance::Image",
            title="update Hypervisor Type", type="string",
            enum=["xen", "qemu", "kvm", "lxc", "uml", "vmware", "hyperv"],
            name="OS::Glance::Image")

    def test_create_namespace_property_with_str_body(self):
        self._test_create_namespace_property()

    def test_create_namespace_property_with_bytes_body(self):
        self._test_create_namespace_property(bytes_body=True)

    def test_list_namespace_property_with_str_body(self):
        self._test_list_namespace_property()

    def test_list_namespace_property_with_bytes_body(self):
        self._test_list_namespace_property(bytes_body=True)

    def test_show_namespace_property_with_str_body(self):
        self._test_show_namespace_property()

    def test_show_namespace_property_with_bytes_body(self):
        self._test_show_namespace_property(bytes_body=True)

    def test_update_namespace_property_with_str_body(self):
        self._test_update_namespace_property()

    def test_update_namespace_property_with_bytes_body(self):
        self._test_update_namespace_property(bytes_body=True)

    def test_delete_namespace(self):
        self.check_service_client_function(
            self.client.delete_namespace_property,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, namespace="OS::Compute::Hypervisor",
            property_name="OS::Glance::Image", status=204)



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest.lib import decorators


class InstanceActionsTestJSON(base.BaseV2ComputeTest):
    """Test instance actions API"""

    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(InstanceActionsTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(InstanceActionsTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')
        cls.request_id = cls.server.response['x-compute-request-id']

    @decorators.idempotent_id('77ca5cc5-9990-45e0-ab98-1de8fead201a')
    def test_list_instance_actions(self):
        """Test listing actions of the provided server"""
        self.client.reboot_server(self.server['id'], type='HARD')
        waiters.wait_for_server_status(self.client,
                                       self.server['id'], 'ACTIVE')

        body = (self.client.list_instance_actions(self.server['id'])
                ['instanceActions'])
        self.assertEqual(len(body), 2, str(body))
        self.assertEqual(sorted([i['action'] for i in body]),
                         ['create', 'reboot'])

    @decorators.idempotent_id('aacc71ca-1d70-4aa5-bbf6-0ff71470e43c')
    def test_get_instance_action(self):
        """Test getting the action details of the provided server"""
        body = self.client.show_instance_action(
            self.server['id'], self.request_id)['instanceAction']
        self.assertEqual(self.server['id'], body['instance_uuid'])
        self.assertEqual('create', body['action'])


class InstanceActionsV221TestJSON(base.BaseV2ComputeTest):
    """Test instance actions with compute microversion greater than 2.20"""

    create_default_network = True

    min_microversion = '2.21'
    max_microversion = 'latest'

    @classmethod
    def setup_clients(cls):
        super(InstanceActionsV221TestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @decorators.idempotent_id('0a0f85d4-10fa-41f6-bf80-a54fb4aa2ae1')
    def test_get_list_deleted_instance_actions(self):
        """Test listing actions for deleted instance

        Listing actions for deleted instance should succeed and the returned
        actions should contain 'create' and 'delete'.
        """
        server = self.create_test_server(wait_until='ACTIVE')
        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])
        body = (self.client.list_instance_actions(server['id'])
                ['instanceActions'])
        self.assertEqual(len(body), 2, str(body))
        self.assertEqual(sorted([i['action'] for i in body]),
                         ['create', 'delete'])



# Copyright 2018 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types
from tempest.lib.api_schema.response.compute.v2_55 import flavors \
    as flavorsv255

# ****** Schemas changed in microversion 2.61 *****************

# Note(gmann): This is schema for microversion 2.61 which includes the
# Flavor extra_specs in the Response body of the following APIs:
#    - ``PUT /flavors/{flavor_id}``
#    - ``GET /flavors/detail``
#    - ``GET /flavors/{flavor_id}``
#    - ``POST /flavors``

flavor_description = {
    'type': ['string', 'null'],
    'minLength': 0, 'maxLength': 65535
}

flavor_extra_specs = {
    'type': 'object',
    'patternProperties': {
        '^[a-zA-Z0-9-_:. ]{1,255}$': {'type': 'string'}
    }
}

common_flavor_info = {
    'type': 'object',
    'properties': {
        'name': {'type': 'string'},
        'links': parameter_types.links,
        'ram': {'type': 'integer'},
        'vcpus': {'type': 'integer'},
        # 'swap' attributes comes as integer value but if it is empty
        # it comes as "". So defining type of as string and integer.
        'swap': {'type': ['integer', 'string']},
        'disk': {'type': 'integer'},
        'id': {'type': 'string'},
        'OS-FLV-DISABLED:disabled': {'type': 'boolean'},
        'os-flavor-access:is_public': {'type': 'boolean'},
        'rxtx_factor': {'type': 'number'},
        'OS-FLV-EXT-DATA:ephemeral': {'type': 'integer'},
        'description': flavor_description,
        'extra_specs': flavor_extra_specs
    },
    'additionalProperties': False,
    # 'OS-FLV-DISABLED', 'os-flavor-access', 'rxtx_factor' and
    # 'OS-FLV-EXT-DATA' are API extensions. so they are not 'required'.
    'required': ['name', 'links', 'ram', 'vcpus', 'swap', 'disk', 'id',
                 'description']
}

list_flavors_details = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'flavors': {
                'type': 'array',
                'items': common_flavor_info
            },
            # NOTE(gmann): flavors_links attribute is not necessary
            # to be present always so it is not 'required'.
            'flavors_links': parameter_types.links
        },
        'additionalProperties': False,
        'required': ['flavors']
    }
}

create_update_get_flavor_details = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'flavor': common_flavor_info
        },
        'additionalProperties': False,
        'required': ['flavor']
    }
}

# ****** Schemas unchanged in microversion 2.61 since microversion 2.55 ***
# Note(gmann): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.55 ***
list_flavors = copy.deepcopy(flavorsv255.list_flavors)

# ****** Schemas unchanged since microversion 2.1 ***
delete_flavor = copy.deepcopy(flavorsv255.delete_flavor)



# Copyright 2014 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.lib.cli import output_parser
from tempest.lib import exceptions
from tempest.tests import base


class TestOutputParser(base.TestCase):
    OUTPUT_LINES = """
+----+------+---------+
| ID | Name | Status  |
+----+------+---------+
| 11 | foo  | BUILD   |
| 21 | bar  | ERROR   |
| 31 | bee  | None    |
+----+------+---------+
"""
    OUTPUT_LINES2 = """
+----+-------+---------+
| ID | Name2 | Status2 |
+----+-------+---------+
| 41 | aaa   | SSSSS   |
| 51 | bbb   | TTTTT   |
| 61 | ccc   | AAAAA   |
+----+-------+---------+
"""

    EXPECTED_TABLE = {'headers': ['ID', 'Name', 'Status'],
                      'values': [['11', 'foo', 'BUILD'],
                                 ['21', 'bar', 'ERROR'],
                                 ['31', 'bee', 'None']]}
    EXPECTED_TABLE2 = {'headers': ['ID', 'Name2', 'Status2'],
                       'values': [['41', 'aaa', 'SSSSS'],
                                  ['51', 'bbb', 'TTTTT'],
                                  ['61', 'ccc', 'AAAAA']]}

    def test_table_with_normal_values(self):
        actual = output_parser.table(self.OUTPUT_LINES)
        self.assertIsInstance(actual, dict)
        self.assertEqual(self.EXPECTED_TABLE, actual)

    def test_table_with_list(self):
        output_lines = self.OUTPUT_LINES.split('\n')
        actual = output_parser.table(output_lines)
        self.assertIsInstance(actual, dict)
        self.assertEqual(self.EXPECTED_TABLE, actual)

    def test_table_with_invalid_line(self):
        output_lines = self.OUTPUT_LINES + "aaaa"
        actual = output_parser.table(output_lines)
        self.assertIsInstance(actual, dict)
        self.assertEqual(self.EXPECTED_TABLE, actual)

    def test_tables_with_normal_values(self):
        output_lines = ('test' + self.OUTPUT_LINES +
                        'test2' + self.OUTPUT_LINES2)
        expected = [{'headers': self.EXPECTED_TABLE['headers'],
                     'label': 'test',
                     'values': self.EXPECTED_TABLE['values']},
                    {'headers': self.EXPECTED_TABLE2['headers'],
                     'label': 'test2',
                     'values': self.EXPECTED_TABLE2['values']}]
        actual = output_parser.tables(output_lines)
        self.assertIsInstance(actual, list)
        self.assertEqual(expected, actual)

    def test_tables_with_invalid_values(self):
        output_lines = ('test' + self.OUTPUT_LINES +
                        'test2' + self.OUTPUT_LINES2 + '\n')
        expected = [{'headers': self.EXPECTED_TABLE['headers'],
                     'label': 'test',
                     'values': self.EXPECTED_TABLE['values']},
                    {'headers': self.EXPECTED_TABLE2['headers'],
                     'label': 'test2',
                     'values': self.EXPECTED_TABLE2['values']}]
        actual = output_parser.tables(output_lines)
        self.assertIsInstance(actual, list)
        self.assertEqual(expected, actual)

    def test_tables_with_invalid_line(self):
        output_lines = ('test' + self.OUTPUT_LINES +
                        'test2' + self.OUTPUT_LINES2 +
                        '+----+-------+---------+')
        expected = [{'headers': self.EXPECTED_TABLE['headers'],
                     'label': 'test',
                     'values': self.EXPECTED_TABLE['values']},
                    {'headers': self.EXPECTED_TABLE2['headers'],
                     'label': 'test2',
                     'values': self.EXPECTED_TABLE2['values']}]

        actual = output_parser.tables(output_lines)
        self.assertIsInstance(actual, list)
        self.assertEqual(expected, actual)

    LISTING_OUTPUT = """
+----+
| ID |
+----+
| 11 |
| 21 |
| 31 |
+----+
"""

    def test_listing(self):
        expected = [{'ID': '11'}, {'ID': '21'}, {'ID': '31'}]
        actual = output_parser.listing(self.LISTING_OUTPUT)
        self.assertIsInstance(actual, list)
        self.assertEqual(expected, actual)

    def test_details_multiple_with_invalid_line(self):
        self.assertRaises(exceptions.InvalidStructure,
                          output_parser.details_multiple,
                          self.OUTPUT_LINES)

    DETAILS_LINES1 = """First Table
+----------+--------+
| Property | Value  |
+----------+--------+
| foo      | BUILD  |
| bar      | ERROR  |
| bee      | None   |
+----------+--------+
"""
    DETAILS_LINES2 = """Second Table
+----------+--------+
| Property | Value  |
+----------+--------+
| aaa      | VVVVV  |
| bbb      | WWWWW  |
| ccc      | XXXXX  |
+----------+--------+
"""

    def test_details_with_normal_line_label_false(self):
        expected = {'foo': 'BUILD', 'bar': 'ERROR', 'bee': 'None'}
        actual = output_parser.details(self.DETAILS_LINES1)
        self.assertEqual(expected, actual)

    def test_details_with_normal_line_label_true(self):
        expected = {'__label': 'First Table',
                    'foo': 'BUILD', 'bar': 'ERROR', 'bee': 'None'}
        actual = output_parser.details(self.DETAILS_LINES1, with_label=True)
        self.assertEqual(expected, actual)

    def test_details_multiple_with_normal_line_label_false(self):
        expected = [{'foo': 'BUILD', 'bar': 'ERROR', 'bee': 'None'},
                    {'aaa': 'VVVVV', 'bbb': 'WWWWW', 'ccc': 'XXXXX'}]
        actual = output_parser.details_multiple(self.DETAILS_LINES1 +
                                                self.DETAILS_LINES2)
        self.assertIsInstance(actual, list)
        self.assertEqual(expected, actual)

    def test_details_multiple_with_normal_line_label_true(self):
        expected = [{'__label': 'First Table',
                     'foo': 'BUILD', 'bar': 'ERROR', 'bee': 'None'},
                    {'__label': 'Second Table',
                     'aaa': 'VVVVV', 'bbb': 'WWWWW', 'ccc': 'XXXXX'}]
        actual = output_parser.details_multiple(self.DETAILS_LINES1 +
                                                self.DETAILS_LINES2,
                                                with_label=True)
        self.assertIsInstance(actual, list)
        self.assertEqual(expected, actual)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from tempest.api.image import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class BasicOperationsImagesAdminTest(base.BaseV2ImageAdminTest):
    """"Test image operations about image owner"""

    @decorators.related_bug('1420008')
    @decorators.idempotent_id('646a6eaa-135f-4493-a0af-12583021224e')
    def test_create_image_owner_param(self):
        """Test creating image with specified owner"""
        # NOTE: Create image with owner different from tenant owner by
        # using "owner" parameter requires an admin privileges.
        random_id = data_utils.rand_uuid_hex()
        image = self.admin_client.create_image(
            container_format='bare', disk_format='raw', owner=random_id)
        self.addCleanup(self.admin_client.delete_image, image['id'])
        image_info = self.admin_client.show_image(image['id'])
        self.assertEqual(random_id, image_info['owner'])

    @decorators.related_bug('1420008')
    @decorators.idempotent_id('525ba546-10ef-4aad-bba1-1858095ce553')
    def test_update_image_owner_param(self):
        """Test updating image owner"""
        random_id_1 = data_utils.rand_uuid_hex()
        image = self.admin_client.create_image(
            container_format='bare', disk_format='raw', owner=random_id_1)
        self.addCleanup(self.admin_client.delete_image, image['id'])
        created_image_info = self.admin_client.show_image(image['id'])

        random_id_2 = data_utils.rand_uuid_hex()
        self.admin_client.update_image(
            image['id'], [dict(replace="/owner", value=random_id_2)])
        updated_image_info = self.admin_client.show_image(image['id'])

        self.assertEqual(random_id_2, updated_image_info['owner'])
        self.assertNotEqual(created_image_info['owner'],
                            updated_image_info['owner'])


class ImportCopyImagesTest(base.BaseV2ImageAdminTest):
    """Test the import copy-image operations"""

    @classmethod
    def skip_checks(cls):
        super(ImportCopyImagesTest, cls).skip_checks()
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @decorators.idempotent_id('9b3b644e-03d1-11eb-a036-fa163e2eaf49')
    def test_image_copy_image_import(self):
        """Test 'copy-image' import functionalities

        Create image, import image with copy-image method and
        verify that import succeeded.
        """
        available_stores = self.get_available_stores()
        available_import_methods = self.client.info_import()[
            'import-methods']['value']
        # NOTE(gmann): Skip if copy-image import method and multistore
        # are not available.
        if ('copy-image' not in available_import_methods or
            not available_stores):
            raise self.skipException('Either copy-image import method or '
                                     'multistore is not available')
        uuid = data_utils.rand_uuid()
        image_name = data_utils.rand_name('copy-image')
        container_format = CONF.image.container_formats[0]
        disk_format = CONF.image.disk_formats[0]
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  visibility='private',
                                  ramdisk_id=uuid)
        self.assertEqual('queued', image['status'])

        file_content = data_utils.random_bytes()
        image_file = io.BytesIO(file_content)
        self.client.store_image_file(image['id'], image_file)

        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual(len(file_content), body.get('size'))
        self.assertEqual('active', body['status'])

        # Copy image to all the stores. In case of all_stores request
        # glance will skip the stores where image is already available.
        self.admin_client.image_import(image['id'], method='copy-image',
                                       all_stores=True,
                                       all_stores_must_succeed=False)

        # Wait for copy to finished on all stores.
        failed_stores = waiters.wait_for_image_copied_to_stores(
            self.client, image['id'])
        # Assert if copy is failed on any store.
        self.assertEqual(0, len(failed_stores),
                         "Failed to copy the following stores: %s" %
                         str(failed_stores))



# Copyright 2018 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

from oslo_serialization import jsonutils as json

from tempest.lib.services.volume.v3 import services_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestServicesClient(base.BaseServiceTest):

    FAKE_SERVICE_LIST = {
        "services": [
            {
                "status": "enabled",
                "binary": "cinder-backup",
                "zone": "nova",
                "state": "up",
                "updated_at": "2017-07-20T07:20:17.000000",
                "host": "fake-host",
                "disabled_reason": None
            },
            {
                "status": "enabled",
                "binary": "cinder-scheduler",
                "zone": "nova",
                "state": "up",
                "updated_at": "2017-07-20T07:20:24.000000",
                "host": "fake-host",
                "disabled_reason": None
            },
            {
                "status": "enabled",
                "binary": "cinder-volume",
                "zone": "nova",
                "frozen": False,
                "state": "up",
                "updated_at": "2017-07-20T07:20:20.000000",
                "host": "fake-host@lvm",
                "replication_status": "disabled",
                "active_backend_id": None,
                "disabled_reason": None
            }
        ]
    }

    FAKE_SERVICE_REQUEST = {
        "host": "fake-host",
        "binary": "cinder-volume"
    }

    FAKE_SERVICE_RESPONSE = {
        "disabled": False,
        "status": "enabled",
        "host": "fake-host@lvm",
        "service": "",
        "binary": "cinder-volume",
        "disabled_reason": None
    }

    def setUp(self):
        super(TestServicesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = services_client.ServicesClient(fake_auth,
                                                     'volume',
                                                     'regionOne')

    def _test_list_services(self, bytes_body=False,
                            mock_args='os-services', **params):
        self.check_service_client_function(
            self.client.list_services,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVICE_LIST,
            to_utf=bytes_body,
            mock_args=[mock_args],
            **params)

    def _test_enable_service(self, bytes_body=False):
        resp_body = self.FAKE_SERVICE_RESPONSE
        kwargs = self.FAKE_SERVICE_REQUEST
        payload = json.dumps(kwargs, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(services_client.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.enable_service,
                'tempest.lib.common.rest_client.RestClient.put',
                resp_body,
                to_utf=bytes_body,
                mock_args=['os-services/enable', payload],
                **kwargs)

    def _test_disable_service(self, bytes_body=False):
        resp_body = copy.deepcopy(self.FAKE_SERVICE_RESPONSE)
        resp_body.pop('disabled_reason')
        resp_body['disabled'] = True
        resp_body['status'] = 'disabled'
        kwargs = self.FAKE_SERVICE_REQUEST
        payload = json.dumps(kwargs, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(services_client.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.disable_service,
                'tempest.lib.common.rest_client.RestClient.put',
                resp_body,
                to_utf=bytes_body,
                mock_args=['os-services/disable', payload],
                **kwargs)

    def _test_disable_log_reason(self, bytes_body=False):
        resp_body = copy.deepcopy(self.FAKE_SERVICE_RESPONSE)
        resp_body['disabled_reason'] = "disabled for test"
        resp_body['disabled'] = True
        resp_body['status'] = 'disabled'
        kwargs = copy.deepcopy(self.FAKE_SERVICE_REQUEST)
        kwargs.update({"disabled_reason": "disabled for test"})
        payload = json.dumps(kwargs, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(services_client.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.disable_log_reason,
                'tempest.lib.common.rest_client.RestClient.put',
                resp_body,
                to_utf=bytes_body,
                mock_args=['os-services/disable-log-reason', payload],
                **kwargs)

    def _test_freeze_host(self, bytes_body=False):
        kwargs = {'host': 'host1@lvm'}
        self.check_service_client_function(
            self.client.freeze_host,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            **kwargs)

    def _test_thaw_host(self, bytes_body=False):
        kwargs = {'host': 'host1@lvm'}
        self.check_service_client_function(
            self.client.thaw_host,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            **kwargs)

    def test_list_services_with_str_body(self):
        self._test_list_services()

    def test_list_services_with_bytes_body(self):
        self._test_list_services(bytes_body=True)

    def test_list_services_with_params(self):
        mock_args = 'os-services?host=fake-host'
        self._test_list_services(mock_args=mock_args, host='fake-host')

    def test_enable_service_with_str_body(self):
        self._test_enable_service()

    def test_enable_service_with_bytes_body(self):
        self._test_enable_service(bytes_body=True)

    def test_disable_service_with_str_body(self):
        self._test_disable_service()

    def test_disable_service_with_bytes_body(self):
        self._test_disable_service(bytes_body=True)

    def test_disable_log_reason_with_str_body(self):
        self._test_disable_log_reason()

    def test_disable_log_reason_with_bytes_body(self):
        self._test_disable_log_reason(bytes_body=True)

    def test_freeze_host_with_str_body(self):
        self._test_freeze_host()

    def test_freeze_host_with_bytes_body(self):
        self._test_freeze_host(bytes_body=True)

    def test_thaw_host_with_str_body(self):
        self._test_thaw_host()

    def test_thaw_host_with_bytes_body(self):
        self._test_thaw_host(bytes_body=True)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class FlavorsExtraSpecsNegativeTestJSON(base.BaseV2ComputeAdminTest):
    """Negative Tests Flavor Extra Spec API extension.

    SET, UNSET, UPDATE Flavor Extra specs require admin privileges.
    """

    @classmethod
    def skip_checks(cls):
        super(FlavorsExtraSpecsNegativeTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('OS-FLV-EXT-DATA', 'compute'):
            msg = "OS-FLV-EXT-DATA extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(FlavorsExtraSpecsNegativeTestJSON, cls).resource_setup()

        flavor_name = data_utils.rand_name('test_flavor')
        ram = 512
        vcpus = 1
        disk = 10
        ephemeral = 10
        new_flavor_id = data_utils.rand_int_id(start=1000)
        swap = 1024
        rxtx = 1
        # Create a flavor
        cls.flavor = cls.admin_flavors_client.create_flavor(
            name=flavor_name,
            ram=ram, vcpus=vcpus,
            disk=disk,
            id=new_flavor_id,
            ephemeral=ephemeral,
            swap=swap,
            rxtx_factor=rxtx)['flavor']
        cls.addClassResourceCleanup(
            cls.admin_flavors_client.wait_for_resource_deletion,
            cls.flavor['id'])
        cls.addClassResourceCleanup(cls.admin_flavors_client.delete_flavor,
                                    cls.flavor['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a00a3b81-5641-45a8-ab2b-4a8ec41e1d7d')
    def test_flavor_non_admin_set_keys(self):
        """Test to SET flavor extra spec as a user without admin privileges"""
        self.assertRaises(lib_exc.Forbidden,
                          self.flavors_client.set_flavor_extra_spec,
                          self.flavor['id'],
                          **{'hw:numa_nodes': '1', 'hw:cpu_policy': 'shared'})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1ebf4ef8-759e-48fe-a801-d451d80476fb')
    def test_flavor_non_admin_update_specific_key(self):
        """non admin user is not allowed to update flavor extra spec"""
        body = self.admin_flavors_client.set_flavor_extra_spec(
            self.flavor['id'],
            **{'hw:numa_nodes': '1', 'hw:cpu_policy': 'shared'}
        )['extra_specs']
        self.assertEqual(body['hw:numa_nodes'], '1')
        self.assertRaises(lib_exc.Forbidden,
                          self.flavors_client.
                          update_flavor_extra_spec,
                          self.flavor['id'],
                          'hw:numa_nodes',
                          **{'hw:numa_nodes': '1'})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('28f12249-27c7-44c1-8810-1f382f316b11')
    def test_flavor_non_admin_unset_keys(self):
        """non admin user is not allowed to unset flavor extra spec"""
        self.admin_flavors_client.set_flavor_extra_spec(
            self.flavor['id'],
            **{'hw:numa_nodes': '1', 'hw:cpu_policy': 'shared'}
        )

        self.assertRaises(lib_exc.Forbidden,
                          self.flavors_client.unset_flavor_extra_spec,
                          self.flavor['id'],
                          'hw:numa_nodes')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('440b9f3f-3c7f-4293-a106-0ceda350f8de')
    def test_flavor_unset_nonexistent_key(self):
        """Unsetting non existence flavor extra spec key should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.admin_flavors_client.unset_flavor_extra_spec,
                          self.flavor['id'],
                          'hw:cpu_thread_policy')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('329a7be3-54b2-48be-8052-bf2ce4afd898')
    def test_flavor_get_nonexistent_key(self):
        """Getting non existence flavor extra spec key should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.flavors_client.show_flavor_extra_spec,
                          self.flavor['id'],
                          'hw:cpu_thread_policy')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('25b822b8-9f49-44f6-80de-d99f0482e5cb')
    def test_flavor_update_mismatch_key(self):
        """Updating unmatched flavor extra spec key should fail

        The key to be updated should match the key in the body
        """
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_flavors_client.update_flavor_extra_spec,
                          self.flavor['id'],
                          'hw:numa_nodes',
                          **{'hw:cpu_policy': 'shared'})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f5889590-bf66-41cc-b4b1-6e6370cfd93f')
    def test_flavor_update_more_key(self):
        """Updating multiple flavor spec keys should fail

        There should be just one item in the request body
        """
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_flavors_client.update_flavor_extra_spec,
                          self.flavor['id'],
                          'hw:numa_nodes',
                          **{'hw:numa_nodes': '1', 'hw:cpu_policy': 'shared'})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Utility for listing all currently installed Tempest plugins.

**Usage:** ``tempest list-plugins``.
"""

from cliff import command
import prettytable

from tempest.test_discover import plugins as plg


class TempestListPlugins(command.Command):
    def take_action(self, parsed_args):
        self._list_plugins()

    def get_description(self):
        return 'List all tempest plugins'

    def _list_plugins(self):
        plugins = plg.TempestTestPluginManager()

        output = prettytable.PrettyTable(["Name", "EntryPoint"])
        for plugin in plugins.ext_plugins.extensions:
            output.add_row([
                plugin.name, plugin.entry_point_target])

        print(output)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

_server_usages = {
    'type': 'array',
    'items': {
        'type': 'object',
        'properties': {
            'ended_at': parameter_types.date_time_or_null,
            'flavor': {'type': 'string'},
            'hours': {'type': 'number'},
            'instance_id': {'type': 'string'},
            'local_gb': {'type': 'integer'},
            'memory_mb': {'type': 'integer'},
            'name': {'type': 'string'},
            'started_at': parameter_types.date_time,
            'state': {'type': 'string'},
            'tenant_id': {'type': 'string'},
            'uptime': {'type': 'integer'},
            'vcpus': {'type': 'integer'},
        },
        'required': ['ended_at', 'flavor', 'hours', 'instance_id', 'local_gb',
                     'memory_mb', 'name', 'started_at', 'state', 'tenant_id',
                     'uptime', 'vcpus']
    }
}

_tenant_usage_list = {
    'type': 'object',
    'properties': {
        'server_usages': _server_usages,
        'start': parameter_types.date_time,
        'stop': parameter_types.date_time,
        'tenant_id': {'type': 'string'},
        'total_hours': {'type': 'number'},
        'total_local_gb_usage': {'type': 'number'},
        'total_memory_mb_usage': {'type': 'number'},
        'total_vcpus_usage': {'type': 'number'},
    },
    'required': ['start', 'stop', 'tenant_id',
                 'total_hours', 'total_local_gb_usage',
                 'total_memory_mb_usage', 'total_vcpus_usage']
}

# 'required' of get_tenant is different from list_tenant's.
_tenant_usage_get = copy.deepcopy(_tenant_usage_list)
_tenant_usage_get['required'] = ['server_usages', 'start', 'stop', 'tenant_id',
                                 'total_hours', 'total_local_gb_usage',
                                 'total_memory_mb_usage', 'total_vcpus_usage']

list_tenant_usage = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'tenant_usages': {
                'type': 'array',
                'items': _tenant_usage_list
            }
        },
        'required': ['tenant_usages']
    }
}

get_tenant_usage = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'tenant_usage': _tenant_usage_get
        },
        'required': ['tenant_usage']
    }
}
